/*
* main.cpp - Proof of concept for a Delphi Obfuscator
*
* Copyright (c) 2005 Sebastian Porst (webmaster@the-interweb.com)
* All rights reserved.
*
* This software is licensed under the zlib/libpng License.
* For more details see http://www.opensource.org/licenses/zlib-license.php
*/

#include "DFMParser.h"
#include "VmtDir.h"
#include "helpers.h"
#include "obfuscate.h"
#include "write.h"
#include "sync.h"

#include <cstdlib>
#include <iostream>
#include <string>
#include <fstream>
#include <ctime>
#include <map>
#include <PeLib.h>

extern unsigned int g_recognizedVmts;

void printUsage()
{
	std::cout << "Usage: pythia.exe file\n";
}

void printStats()
{
	std::cout << "Recognized VMTs: " << g_recognizedVmts << std::endl;
}

int main(int argc, char *argv[])
{
	std::cout << "Delphi Obfuscator 1.0 - Author: Sebastian Porst (webmaster@the-interweb.com)\n\n";
	
	srand(time(0));
	
	if (argc != 2)
	{
		printUsage();
		return 1;
	}
	
    std::string filename = argv[1];
    
    PeLib::PeFile32 pefile(filename);
    
    if (pefile.readMzHeader() == 0 && pefile.readPeHeader() == 0 && pefile.readResourceDirectory() == 0)
    {
	    VMTDir vmtdir;
		
		readVMTs(pefile, vmtdir);
		
	    DFMData dfmresources;
	    
	    try
	    {
		    readDFMResources(pefile, dfmresources);
			synchronize(dfmresources, vmtdir);
			obfuscate(dfmresources, vmtdir);
			store(filename, dfmresources, vmtdir, pefile);
		}
		catch(const std::string& e)
		{
			die(e);
		}
		
		printStats();
		
		std::cout << "Everything seems to have worked. Try to start the obfuscated file now." << std::endl;
		
	    return EXIT_SUCCESS;
	}
	else
	{
		die("Error: File does not seem to be a valid Delphi file.");
	}
}
